#include <LSM303.h>

#include <Event.h>
#include <Timer.h>
#include <Adafruit_NeoPixel.h>
#include <Servo.h>
#include <AStar32U4.h>
#include <PololuRPiSlave.h>

struct Data
{
  bool yellow, green, red;
  bool buttonA, buttonB, buttonC;
  int16_t leftMotor, rightMotor;
  uint16_t batteryMillivolts;
  uint8_t pan_pos,tilt_pos,radar_pos;
  uint16_t compass_reading;
};

bool led_green_state = false;
Timer t;
Adafruit_NeoPixel neo(10, 8, NEO_GRB + NEO_KHZ800);
PololuRPiSlave<struct Data,5> slave;
AStar32U4Motors motors;
AStar32U4ButtonA buttonA;
AStar32U4ButtonB buttonB;
AStar32U4ButtonC buttonC;
Servo servo_pan,servo_tilt,servo_radar;

void blink_green()
{
    ledGreen(led_green_state);
    led_green_state = !led_green_state;
}

void write_debug_data()
{
  Serial.println("Zero debug data:");
  Serial.print("\tBattery: ");
  Serial.print(slave.buffer.batteryMillivolts);
  Serial.println(" mV");
  Serial.print("\tLeft moto: ");
  Serial.println(slave.buffer.leftMotor);
  Serial.print("\tRight moto: ");
  Serial.println(slave.buffer.rightMotor);
}

void setup()
{
  Serial.begin(115200);
  slave.init(0x20);  
  pinMode(13, OUTPUT);
  t.every(500, blink_green);
  t.every(2000, write_debug_data);
}

void loop()
{
  // Call updateBuffer() before using the buffer, to get the latest
  // data including recent master writes.
  slave.updateBuffer();

  // Write various values into the data structure.
  slave.buffer.buttonA = buttonA.isPressed();
  slave.buffer.buttonB = buttonB.isPressed();
  slave.buffer.buttonC = buttonC.isPressed();
  slave.buffer.pan_pos = servo_pan.read();
  slave.buffer.tilt_pos = servo_tilt.read();
  slave.buffer.radar_pos = servo_radar.read();
  
  // Change this to readBatteryMillivoltsLV() for the LV model.
  slave.buffer.batteryMillivolts = readBatteryMillivoltsLV();

  // READING the buffer is allowed before or after finalizeWrites().
  ledYellow(slave.buffer.yellow);
  ledGreen(slave.buffer.green);
  ledRed(slave.buffer.red);
  motors.setSpeeds(slave.buffer.leftMotor, slave.buffer.rightMotor);  

  // When you are done WRITING, call finalizeWrites() to make modified
  // data available to I2C master.
  slave.finalizeWrites();
  t.update();
}
